class Attribution:
    # region Title
    # The Script_Title attribute should be set in your implementation
    attribute_title = None

    @classmethod
    def get_title(cls):
        """
        Returns a set title

        Raises:
            NotImplementedError: There is no title defined, please define it as 'attribute_title='

        Returns:
            str: The title
        """
        if cls.attribute_title is None:
            raise NotImplementedError("There is no title defined, please define it as 'attribute_title='")
        return cls.attribute_title
    # endregion

    # region Description
    # The attribute_description attribute should be set in your implementation
    attribute_description = None

    @classmethod
    def get_description(cls):
        """
        Returns a description

        Raises:
            NotImplementedError: There is no description defined, please define it as 'attribute_description='

        Returns:
            str: The description
        """
        if cls.attribute_description is None:
            raise NotImplementedError("There is no description defined, please define it as 'attribute_description='")
        return cls.attribute_description
    # endregion

    # region Author
    # The attribute_author attribute should be set in your implementation
    attribute_author = None

    @classmethod
    def get_author(cls):
        """
        Returns the author

        Raises:
            NotImplementedError: There is no author defined, please define it as 'attribute_author='

        Returns:
            str: The author
        """
        if cls.attribute_author is None:
            raise NotImplementedError("There is no author defined, please define it as 'attribute_author='")
        return cls.attribute_author
    # endregion

    # region Owner
    # The attribute_owner attribute should be set in your implementation
    attribute_owner = None

    @classmethod
    def get_owner(cls):
        """
        Returns a owner

        Raises:
            NotImplementedError: There is no owner defined, please define it as 'attribute_owner='

        Returns:
            str: The owner
        """
        if cls.attribute_owner is None:
            raise NotImplementedError("There is no owner defined, please define it as 'attribute_owner='")
        return cls.attribute_owner
    # endregion

    # region Support_Link
    # The attribute_supportLink attribute should be set in your implementation
    attribute_supportLink = None

    @classmethod
    def get_support_link(cls):
        """
        Returns a support link

        Raises:
            NotImplementedError: There is no supportLink defined, please define it as 'attribute_supportLink='

        Returns:
            str: The support link
        """
        if cls.attribute_supportLink is None:
            raise NotImplementedError("There is no supportLink defined, please define it as 'attribute_supportLink='")
        return cls.attribute_supportLink
    # endregion

    # region Docs_Link
    # The attribute_docs_link attribute should be set in your implementation
    attribute_docs_link = None

    @classmethod
    def get_docs_link(cls):
        """
        Returns a doc link

        Raises:
            NotImplementedError: There is no DocLink defined, please define it as 'attribute_docs_link='

        Returns:
            str: The doc link
        """
        if cls.attribute_docs_link is None:
            raise NotImplementedError("There is no DocLink defined, please define it as 'attribute_docs_link='")
        return cls.attribute_docs_link
    # endregion

    # region LicenseLink
    # The attribute_license_link attribute should be set in your implementation
    attribute_license_link = None

    @classmethod
    def get_license_link(cls):
        """
        Returns a license link

        Raises:
            NotImplementedError: There is no LicenseLink defined, please define it as 'attribute_license_link='

        Returns:
            str: The license link
        """
        if cls.attribute_license_link is None:
            raise NotImplementedError("There is no LicenseLink defined, please define it as 'attribute_license_link='")
        return cls.attribute_license_link
    # endregion



#!/usr/bin/env python3
import argparse
import importlib
import json
import os
import re
import sys
from datetime import datetime
from pathlib import Path, PurePath
from typing import List, Optional

import toml
from dotenv import load_dotenv

import seethrufeeds.core.config_parser as ConfigParser
from seethrufeeds.model.Feeds import feed
from seethrufeeds.model.Scripts.script_base import ScriptBase
from seethrufeeds.model.Scripts.script_result import ScriptResult
from seethrufeeds.model.Scripts.script_state import StateEngine


class ProgramArgument:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.extra = kwargs


def touch_file(path: str):
    """
    Touches a new file
    Args:
        path: The path to touch the file
    """
    path = os.path.join(path)
    f = open(path, "w")
    f.close()


def create_dir(path: os.path):
    """
    Creates a directory
    Args:
        path: The path of the new directory
    """
    path = os.path.join(path)
    os.mkdir(path)


def get_config_attribute(attribute: str) -> str:
    """
    Replaces all occurrences of $(variable_name) in an attribute with the associated environment variable
    Args:
        attribute: The attribute value

    Returns:
        The attribute
    """
    if type(attribute) != str:
        return attribute
    parts = re.findall(r"\$\([\w\d\-\+]+\)", attribute)
    for arg in parts:
        environment_variable = os.environ.get(arg[2:-1], "")
        attribute = attribute.replace(arg, environment_variable)
    return attribute


class SeeThruFeed:
    Programs = {}

    base_dir: Path
    argv: List[str]
    config_file: Path
    config_method: str

    @classmethod
    def setup_programs(cls):
        cls.Programs = {
            "help": {
                "procedure": SeeThruFeed.display_help,
                "arguments": [],
                "uses_config": False,
                "help": "Displays this help screen"
            },
            "createfeedscheme": {
                "procedure": SeeThruFeed.create_feedscheme,
                "arguments": [
                    ProgramArgument("name", action="store",
                                    type=str, help="The name of the scheme"),
                    ProgramArgument("--path", "-p", action="store", type=str, required=False,
                                    help="The directory to store the feedscheme, defaults to ./name"),
                    ProgramArgument("--no-config", action="store_const", const=True, required=False,
                                    help="Create a scheme without a config file"),
                    ProgramArgument("--config_path", action="store", type=str, required=False,
                                    help="The name of the script to use for the feed"),
                    ProgramArgument("--config_method", action="store", choices=["json", "toml"], type=str, 
                                    required=False, help="The name of the script to use for the feed")
                ],
                "uses_config": False,
                "help": "Create a feed scheme with the given name, this will create the feedscheme in a new directory"
            },
            "createscript": {
                "procedure": SeeThruFeed.create_script,
                "arguments": [
                    ProgramArgument("name", action="store",
                                    type=str, help="The name of the script"),
                    ProgramArgument("--no-config", action="store_const", const=True, required=False,
                                    help="Create the script without a config file")
                ],
                "uses_config": True,
                "help": "Creates a new script in the Scripts/ directory as well as a config file entry"
            },
            "addscript": {
                "procedure": SeeThruFeed.add_script,
                "arguments": [
                    ProgramArgument("name", action="store",
                                    type=str, help="The name of the script"),
                    ProgramArgument("--script", "-s", action="store", required=False,
                                    help="The object import path of the script file to use")
                ],
                "uses_config": True,
                "help": "Add a pre-existing script into the config file"
            },
            "addscriptstate": {
                "procedure": SeeThruFeed.add_scriptstate,
                "arguments": [
                    ProgramArgument("script", action="store", type=str,
                                    help="The name of the script to add the state to"),
                    ProgramArgument("--name", action="store", type=str, required=True,
                                    help="The name of the state"),
                    ProgramArgument("--status", action="store", type=str, required=True,
                                    help="The status to give the state"),
                    ProgramArgument("--message", action="store", type=str, required=True, default="",
                                    help="The message to give the state")
                ],
                "uses_config": True,
                "help": "Add a state to a script"
            },
            "createfeed": {
                "procedure": SeeThruFeed.create_feed,
                "arguments": [
                    ProgramArgument("name", action="store", type=str,
                                    help="The internal name of the feed"),
                    ProgramArgument("--script", "-s", action="store", type=str, required=True,
                                    help="The name of the script to use for the feed"),
                    ProgramArgument("--api-key", "-k", default="apikey", action="store", type=str, required=True,
                                    help="The reference of the apikey to use for the feed"),
                    ProgramArgument("--guid", "-g", action="store", type=str, required=True,
                                    help="The guid of the feed")
                ],
                "uses_config": True,
                "help": "Enters a feed into the config file, using the given script as a source"
            },
            "addapikey": {
                "procedure": SeeThruFeed.add_apikey,
                "arguments": [
                    ProgramArgument("reference", default="apikey", action="store", type=str,
                                    help="The reference to give the apikey"),
                    ProgramArgument("--no-env", action="store_const", const=True, required=False,
                                    help="Create the apikey without storing the token and secret in an env file"),
                    ProgramArgument("--access-token", "-t", action="store", type=str, required=False,
                                    help="The access token of the apikey"),
                    ProgramArgument("--secret", "-s", action="store", type=str, required=False,
                                    help="The secret of the apikey")
                ],
                "uses_config": True,
                "help": "Adds a new api key to the config file, this can be used by feeds to upload results to SeeThru"
            },
            "convert": {
                "procedure": SeeThruFeed.convert_config,
                "arguments": [
                    ProgramArgument("--to", default="json", action="store", type=str,
                                    help="Convert the config file to a new format"),
                    ProgramArgument("--output", "-o", action="store", type=str, required=False,
                                    help="The output file of the new config")
                ],
                "uses_config": True,
                "help": "Converts the config file from one type to another"
            },
            "runfeedscheme": {
                "procedure": SeeThruFeed.run_feedscheme,
                "arguments": [],
                "uses_config": True,
                "help": "Runs the feed scheme defined in the config file and uploaded any feeds"
            },
            "run": {
                "alias": "runfeedscheme",
                "help": "An alias of runfeedscheme"
            }
        }

    def __init__(self, argv=None, base_dir=os.getcwd()):
        """
        Provides the core functionality

        Args:
            argv (List[str]): Command line arguments, if provided, run is automatically ran
            base_dir (Union[str, Path]): The base directory to work from
        """
        SeeThruFeed.setup_programs()
        self.base_dir = Path(base_dir)
        self.argv = []
        self.config_file = Path()
        self.config_method = ""

        # Loads any present env file
        load_dotenv(dotenv_path=os.path.join(base_dir, ".env"))

        if argv is not None:
            self.run(argv)

    def run(self, argv):
        if len(argv) == 1:  # If no arguments were given, print a help Message
            print("Please provide a valid program: ")
            for program in SeeThruFeed.Programs.keys():
                print(f"\t{program}")
            return

        # Removes the first part of the args
        argv = argv[1:]
        self.argv = argv

        # Runs the appropriate program
        if argv[0] in SeeThruFeed.Programs.keys():
            # Gets the program
            programName = argv[0]
            program = SeeThruFeed.Programs[programName]
            argv = argv[1:]

            if "alias" in program:
                programName = program["alias"]
                program = SeeThruFeed.Programs[programName]

            # Creates the argparse program
            parser = argparse.ArgumentParser(prog=programName)
            # Uses every argument defined in the program
            for argument in program["arguments"]:
                parser.add_argument(*argument.args, **argument.extra)
            # If the program uses the config, generic config arguments are added
            if program["uses_config"]:
                # Config path
                parser.add_argument("--config_path", action="store", type=str, required=False,
                                    help="The name of the script to use for the feed")
                # Config method
                parser.add_argument("--config_method", action="store", choices=["json", "toml"], 
                                    type=str, required=False, help="The name of the script to use for the feed")

            # Parses the argument
            args = vars(parser.parse_args(argv))

            # Automatically validates the config file
            if program["uses_config"]:
                path = args["config_path"]
                if type(path) == str:
                    path = Path(path)
                method = args["config_method"]
                self.validate_config_path(path, method)

                del args["config_path"]
                del args["config_method"]

            # Runs the program
            program["procedure"](self, **args)
            return
        else:
            print("Please provide a valid program: ")
            for program in SeeThruFeed.Programs.keys():
                print(f"\t{program}")

    def get_config_path(self):
        """
        Finds a config file and method in the base directory

        Returns:
            bool: Whether a config was found
        """
        for fileName in self.base_dir.iterdir():
            match = re.search(r"(?:\/|\\|^)(?:conf|config).(json|toml)$", fileName.name)
            if match is None:
                continue
            configMethod = match.group(1)
            self.config_file = fileName
            self.config_method = configMethod
            return True
        return False

    def validate_config_path(self, path, method):
        """
        Confirms that the given path and method are valid, if neither are provided, 
        an attempt is made to automatically discover the path and method,
        if only the path is provided, an attemp it made to automatically discover the method.
        The results are stored in self.config_path and self.config_method respectively

        Args:
            path (Path): The config path
            method (str): The config method
        """
        # Attempts to automatically discover a config file
        if path is None:
            if self.get_config_path():
                return
            raise Exception("No config file could be found")

        # Attempts to discover the method
        if method is None:
            # Identifies the config method
            match = re.search(r".(json|toml)$", path.name)
            if match is None:
                raise Exception("The method couldn't be identified from the config path")            
            method = match.group(1)
            
        # Validates the path and method
        if not isinstance(path, PurePath):
            raise TypeError("Path is invalid type")
        if method not in ["json", "toml"]:
            raise TypeError("Please provide a valid method")

        self.config_file = path
        self.config_method = method


    def display_help(self):
        """
        Prints the help screen
        """
        for program in SeeThruFeed.Programs.keys():
            print(f"\t{program} - {SeeThruFeed.Programs[program]['help']}")

    def create_feedscheme(self, name, path=None, no_config=False, config_path=None, config_method=None):
        """
        Creates a new feed scheme of the given name in the
        current working directory

        Arguments:
            name (str): The name of the feed scheme
            no_config (bool): Whether a config file should be generated for the feedscheme
        """
        # Gets the absolute the path
        if path is None:
            path = name
        path = Path(path)
        if not path.is_absolute():
            path = self.base_dir.joinpath(path)

        # Creates the directory structure
        path.mkdir(parents=True, exist_ok=True)

        path.joinpath("__init__.py").touch()
        path.joinpath("Scripts").mkdir()
        path.joinpath("Scripts/__init__.py").touch()
        path.joinpath("Scripts/Vendor").mkdir()
        path.joinpath("Scripts/Vendor/__init__.py").touch()
        path.joinpath("Components").mkdir()
        path.joinpath("Components/__init__.py").touch()
        path.joinpath("Components/Vendor").mkdir()
        path.joinpath("Components/Vendor/__init__.py").touch()
        path.joinpath("Outputs").mkdir()

        # Creates the new manager
        templatePath = Path(__file__).parent.joinpath("templates/Manage_Template.template")
        with open(templatePath, "r") as managerTemplate:
            template = managerTemplate.read()
            
        with open(path.joinpath("manage.py"), "w") as manager:
            manager.write(template)

        if no_config:
            return

        # Validates the config path
        if config_path is None:
            if config_method is None:
                config_method = "json"
            config_path = path.joinpath("config." + config_method)

        self.validate_config_path(config_path, config_method)
        # Creates a config
        config = ConfigParser.Config.new(name)
        ConfigParser.ConfigParser(self.config_file, self.config_method).set_config(config).save()

    def create_script(self, name, no_config=False):
        """
        Creates a new script and adds the script to the config file

        Args:
            name (str): The name of the script
            no_config (bool): Whether the script should be added to the config file
        """
        # Creates the new script
        with open(Path(__file__).parent.joinpath("templates/Script_Template.template"), 'r') as scriptTemplate:
            template = scriptTemplate.read()
        # Sets the default information
        template = template.replace(r"{{ Script_Name }}", name)
        with open(self.base_dir.joinpath(f"Scripts/{name}.py"), "w") as script:
            script.write(template)

        if no_config:
            return

        # Loads the config file
        with ConfigParser.ConfigParser(self.config_file, self.config_method) as config:
            # Creates a new script
            config.add_script(name)

    def add_script(self, name, script=None):
        with ConfigParser.ConfigParser(self.config_file, self.config_method) as config:
            configScript = config.add_script(name)
            if script is not None:
                configScript.Meta.Script_Object_Path = script

    def add_scriptstate(self, script, name, status, message):
        # Loads the config file
        with ConfigParser.ConfigParser(self.config_file, self.config_method) as config:
            if script not in config.Scripts:
                # TODO: Show error message
                return
            config.Scripts[script].add_state(name, status, message)

    def run_feedscheme(self):
        """
        Runs the feed scheme
        """
        # Opens the config file and parses it
        config = ConfigParser.ConfigParser(self.config_file, self.config_method).load()

        if len(config.Scripts) == 0:
            # TODO: Show error message
            return

        scriptResults = {}

        for key, script in config.Scripts.items():
            scriptName = get_config_attribute(script.Meta.Script_Name)

            # Splits the object's module and the object's name from the Script_Object_Path
            objectComponents = get_config_attribute(script.Meta.Script_Object_Path).split('@')
            if len(objectComponents) != 2:
                # TODO: Show error message
                return
            objectModule, objectName = tuple(objectComponents)

            # Dynamically imports the script
            scriptModule = importlib.import_module(objectModule)
            if objectName not in dir(scriptModule):
                # TODO: Show error message
                return
            scriptClass = getattr(scriptModule, objectName)
            # Instantiates the script
            scriptInstance: ScriptBase = scriptClass()

            # Sets any fillables defined
            for key, fillable in script.Fillables.items():
                # Gets all defined environment variables
                  
                # TODO: Perform conversion on different value types
                value = get_config_attribute(fillable)

                # Assigns the fillable to the script
                scriptInstance.set_property(key, fillable)

            # Gets any state defined
            for key, value in script.States.items():
                if isinstance(scriptInstance, StateEngine):
                    # Configures the state to the script
                    scriptInstance: StateEngine
                    try:
                        scriptInstance.configure_state(key, value.Status, value.Message)
                    except ValueError as _:
                        print("Please provide a valid status of either 'red', 'amber' or 'green'. This state will not be used")

            scriptInstance.set_internal_alias(scriptName)
            # Runs the script
            scriptInstance.set_output_path(script.Meta.Script_Output_Path)
            scriptInstance.run_script()
            scriptInstance.evaluate_script()
            scriptInstance.log_output()
            scriptInstance.export_to_output()

            scriptResults[scriptName] = scriptInstance.get_result()

        # Goes through every feed
        if len(config.Feeds) == 0:
            return
        if len(config.Api_Keys):
            return
        for key, feed in config.Feeds.items():
            if feed.Script not in scriptResults:
                continue
            # Gets the api key
            if feed.Api_Key not in config.Api_Keys:
                continue
            accessToken = get_config_attribute(config.Api_Keys[feed.Api_Key].Access_Token)
            secret = get_config_attribute(config.Api_Keys[feed.Api_Key].Secret)

            result = scriptResults[feed.Script]

            # Pushes the script result
            feed = feed.Feed()
            feed.set_guid(feed.Guid)
            feed.set_api_key(accessToken, secret)
            feed.set_script_result(result)
            result = feed.push()
            if result != "":
                print(f"Feed push failed with message: {result}")

    def create_feed(self, name, script, api_key, guid):
        """
        Creates a new feed entry in the config file
        Args:
            name: The internal name of the feed (Internal use only)
            script: The name of the script that this feed uses
            api_key: The reference to the api key to use
            guid: The guid of the feed
        """
        # Loads the config file
        with ConfigParser.ConfigParser(self.config_file, self.config_method) as config:
            # Adds the feed to the config
            config.add_feed(name, script, api_key, guid)

    def add_apikey(self, reference, no_env, access_token, secret):
        """
        Adds an api key to the config file
        Args:
            reference (str): The reference to give the api key
            no_env (bool): If true, the access_toke and secret aren't required and aren't added to the env file
            access_token (str): The access token of the api key
            secret (str): The secret of the api key
        """
        # Only gets the access token and secret if an env file is wanted to store them in
        if no_env is None:
            if access_token is None:
                access_token = input("Please enter the access token: ")
            if secret is None:
                secret = input("Please enter the secret: ")

        # Loads the config file
        with ConfigParser.ConfigParser(self.config_file, self.config_method) as config:
            # Adds the reference to the env file in the config
            config.add_api_key(
                reference,
                f"$({reference.upper()}_ACCESS_TOKEN)",
                f"$({reference.upper()}_SECRET)"
            )

        # Adds the access token and secret to the env file
        if no_env is None:
            envFile = open(os.path.join(self.base_dir, ".env"), "a")
            envFile.write(f"{reference.upper()}_ACCESS_TOKEN={access_token}\n")
            envFile.write(f"{reference.upper()}_SECRET={secret}\n")

    def convert_config(self, to, output=None):
        if to not in ["json", "toml"]:
            print("Please provide a valid method")
            return

        parser = ConfigParser.ConfigParser(
            self.config_file, self.config_method)
        parser.load()
        parser.method = to
        parser.config_path = Path(output)
        parser.save()


def exec():
    """
    Provides an entry point for the command line utility
    """
    SeeThruFeed(sys.argv)



from seethrufeeds.model.Scripts.script_state import State, DefaultStates, StateEngine
from seethrufeeds.model.Scripts.script_base import ScriptBase
from seethrufeeds.model.Properties.properties import FillableProperty, ResultProperty
import requests, json


class StatusPage(ScriptBase, StateEngine):
    class MessageStates(DefaultStates):
        cannot_load_status = State("cannot_load_status", State.amber, "The statuspage couldn't be loaded")
        ERROR = State("ERROR", State.red, "Partial System Outage")
        WARNING = State("WARNING", State.amber, "Minor Service Outage")
        OK = State("OK", State.green, "All Systems Operational")

    id = FillableProperty("id", required=True, of_type=str)

    def script_run(self):
        try:
            response = requests.get(f"https://{self.id.value}.statuspage.io/api/v2/status.json")
            try:
                statuspage = json.loads(response.content)
                # Gets the status
                self.assert_true("status" in statuspage, self.MessageStates.cannot_load_status)
                statuspage = statuspage["status"]

                self.assert_true("indicator" in statuspage, self.MessageStates.cannot_load_status)
                indicator = statuspage["indicator"]

                if indicator == "none":
                    self.set_state(self.MessageStates.OK)
                    return
                elif indicator == "minor":
                    self.set_state(self.MessageStates.WARNING)
                    return
                elif indicator == "major":
                    self.set_state(self.MessageStates.ERROR)
                else:
                    self.set_state(self.MessageStates.cannot_load_status)

            except:
                self.set_state(self.MessageStates.cannot_load_status)
        except:
            self.set_state(self.MessageStates.cannot_load_status)


if __name__=="__main__":
    page = StatusPage()
    page.configure_state("OK", "green", "Zoom is operational")
    page.configure_state("WARNING", "amber", "There is a minor issue with zoom")
    page.configure_state("ERROR", "red", "There is a major issue with zoom")
    page.id.value = "14qjgk812kgk"
    page.run_script().evaluate_script().log_output()



import unittest
from uuid import UUID

from seethrufeeds.feeds.api_key import ApiKey
from seethrufeeds.feeds.exceptions import InvalidAccessToken, InvalidSecretKey


class ApiKeyTestCase(unittest.TestCase):
    def test_api_key(self):
        api_key = ApiKey(UUID("0e3fa201-59b5-47bb-a875-e379ec3cd724"),
                         "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")
        print(api_key)
        self.assertEqual(str(api_key.access_token), "0e3fa201-59b5-47bb-a875-e379ec3cd724")
        self.assertEqual(str(api_key.secret_key), "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")

        self.assertRaises(InvalidAccessToken, ApiKey, "0e3fa201-59b5-47bb-a875-e379ec3cd724",
                                             "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")

        self.assertRaises(InvalidSecretKey, ApiKey, UUID("0e3fa201-59b5-47bb-a875-e379ec3cd724"), 2)

        self.assertRaises(InvalidSecretKey, ApiKey, UUID("0e3fa201-59b5-47bb-a875-e379ec3cd724"), "asfgbg")


if __name__ == '__main__':
    unittest.main()



import unittest
from datetime import datetime

from seethrufeeds.feeds.feed_result import FeedResult, StatusEnum


class FeedResultTestCase(unittest.TestCase):
    def test_create_result(self):
        result = FeedResult(StatusEnum.ok, "Foo Bar")
        self.assertEqual(result.status, StatusEnum.ok)
        self.assertEqual(result.get_status(), StatusEnum.ok)
        self.assertEqual(result.get_status_value(), "green")
        self.assertEqual(result.message, "Foo Bar")
        self.assertEqual(result.get_message(), "Foo Bar")

        self.assertRaises(TypeError, FeedResult, "green", "Foo Bar")
        self.assertRaises(TypeError, FeedResult, StatusEnum.ok, "Foo Bar", "")

        timestamp = datetime.strptime("2021-02-03 15:15:15", "%Y-%m-%d %H:%M:%S")
        result = FeedResult(StatusEnum.ok, "", timestamp)
        self.assertEqual(result.get_timestamp_str(), "2021-02-03 15:15:15")


if __name__ == '__main__':
    unittest.main()



from seethrufeeds.model.Scripts.script_state import DefaultStates, State, StateEngine
from seethrufeeds.model.Scripts.script_base import ScriptBase
from seethrufeeds.model.Properties.properties import FillableProperty, ResultProperty
from seethrufeeds.library.components.socket import UDPPortOpen as PortOpen
import os


class UDPPortOpen(ScriptBase, StateEngine):
    class MessageStates(DefaultStates):
        port_closed = State("port_closed", State.error, "Cannot create a tcp socket to given host and port")

    host = FillableProperty(name="host", required=True, of_type=str)
    port = FillableProperty(name="port", required=False, of_type=[str, int], default=443)

    is_port_open = ResultProperty(name="is_port_open")
    # Stores the error Message given by UDPPortOpen component
    ERROR_MSG = ResultProperty(name="error_msg", default=None)

    Attr_Title = "UDP Port Open"
    Attr_Description = "A script which tests sending data with a udp socket to a given host and port"
    Attr_Author = "SeeThru Networks"
    Attr_Owner = "SeeThru Networks"

    # ------ Script Overrides ------
    def script_run(self):
        udp_test = PortOpen()\
            .set_property(PortOpen.TARGET_HOST, self.host.value)\
            .set_property(PortOpen.PORT, self.port.value)\
            .run()
        self.is_port_open.value = udp_test.SUCCEEDED.value
        self.assert_true(self.is_port_open.value, self.MessageStates.port_closed)

        # Sets the error Message if any
        error_no = udp_test.get_property(PortOpen.ERROR_NO)
        if error_no:
            self.set_property(self.ERROR_MSG, os.strerror(error_no))



from seethrufeeds.model.components.component_base import ComponentBase
from seethrufeeds.model.Properties.properties import FillableProperty, ResultProperty

from typing import List, Union

from pyzabbix.api import ZabbixAPI


class HistoryValue:
    clock: int
    itemid: str
    ns: int

    def __init__(self, data):
        for key in data.keys():
            if key == "value":
                if "_set_value" in dir(self):
                    self._set_value(data[key])
                    continue
            setattr(self, key, data[key])


class FloatHistory(HistoryValue):
    value: float

    def _set_value(self, value):
        self.value = float(value)


class IntegerHistory(HistoryValue):
    value: int

    def _set_value(self, value):
        self.value = int(value)


class StringHistory(HistoryValue):
    value: str


class TextHistory(HistoryValue):
    id: str
    value: str


class LogHistory(HistoryValue):
    id: str
    logeventid: int
    severity: int
    source: str
    timestamp: int
    value: str


class ZabbixItem:
    itemid: str
    delay: str
    hostid: str
    interfaceid: str
    key_: str
    name: str
    type: int
    url: str
    value_type: int
    allow_traps: int
    authtype: int
    description: str
    error: str
    flags: int
    follow_redirects: int
    headers: dict
    history: str
    http_proxy: str
    inventory_link: int
    ipmi_sensor: str
    jmx_endpoint: str
    lastclock: int
    lastns: int
    lastvalue: str
    logtimefmt: str
    master_itemid: int
    output_format: int
    params: str
    password: str
    post_type: int
    posts: str
    prevvalue: str
    privatekey: str
    publickey: str
    query_fields: list
    request_method: int
    retrieve_method: int
    retrieve_mode: int
    snmp_oid: str
    ssl_cert_file: str
    ssl_key_file: str
    ssl_key_password: str
    state: int
    status: int
    stauts_codes: str
    templateid: str
    timeout: str
    trapper_hosts: str
    trends: str
    units: str
    username: str
    valuemapid: str
    verify_host: int
    verify_peer: int

    value_history: List[Union[FloatHistory, IntegerHistory, StringHistory, TextHistory, LogHistory]]

    def __init__(self, item_data):
        for key in item_data.keys():
            setattr(self, key, item_data[key])
        self.value_history = []


class Zabbix(ComponentBase):
    url = FillableProperty("url", required=True, of_type=str)
    username = FillableProperty("username", required=True, of_type=str)
    password = FillableProperty("password", required=True, of_type=str)
    """
    The host that contains the items
    """
    host = FillableProperty("host", required=True, of_type=str)
    history_count = FillableProperty("history_count", required=False, default=10, of_type=int)

    items = ResultProperty("items")

    Component_Title = "Zabbix Component"
    Component_Description = "Gets the recent history of every item of a zabbix host"
    Component_Author = "SeeThru Networks"
    Component_Owner = "SeeThru Networks"

    def component_execute(self):
        with ZabbixAPI(url=self.url.value, user=self.username.value, password=self.password.value) as zabbix:
            # Gets all of the items for the given host
            # noinspection PyTypeChecker
            api_items = zabbix.do_request("item.get", {
                "output": "extend",
                "filter": {
                    "host": [
                        self.host.value
                    ]
                },
                "sortfield": "name"
            })
            # Converts the items into an internal representation
            self.items.value = []
            # Stores a mapping between id and item
            mapping = {}
            for item in api_items["result"]:
                newItem = ZabbixItem(item)
                self.items.value.append(newItem)
                mapping[newItem.itemid] = newItem

            # Gets the value history for each item
            # noinspection PyTypeChecker
            history = zabbix.do_request("history.get", {
                "output": "extend",
                "history": 0,
                "itemids": [item.itemid for item in self.items.value],
                "sortfield": "clock",
                "sortorder": "DESC",
                "limit": self.history_count.value * len(self.items.value)
            })

            # Adds the history to each item
            for historyValue in history["result"]:
                item = mapping[historyValue["itemid"]]
                if item.value_type == '0':  # Numeric Float
                    item.value_history.append(FloatHistory(historyValue))
                elif item.value_type == '1':  # Character
                    item.value_history.append(StringHistory(historyValue))
                elif item.value_type == '2':  # Log
                    item.value_history.append(LogHistory(historyValue))
                elif item.value_type == '3':  # Numeric unsigned (int)
                    item.value_history.append(IntegerHistory(historyValue))
                elif item.value_type == '4':  # Text
                    item.value_history.append(TextHistory(historyValue))



from typing import Any, Dict

from . import meta as ScriptMeta, state as ScriptState

from seethrufeeds.core.exceptions import config as ConfigExceptions

class Script:
    Meta: "ScriptMeta.Meta"
    Fillables: Dict[str, Any]
    States: Dict[str, "ScriptState.State"]

    def __init__(self,
                 meta: "ScriptMeta",
                 fillables: Dict[str, Any],
                 states: Dict[str, "ScriptState"]):
        self.Meta = meta
        self.Fillables = fillables
        self.States = states

    @staticmethod
    def new(name: str) -> "Script":
        """
            Creates a new script, with default attributes

            Args:
                name (str): The name of the script

            Returns:
                ConfigScript: The new script
            """
        meta = ScriptMeta.Meta.new(name)
        return Script(meta, {}, {})

    def add_fillable(self, name: str, value: Any) -> Any:
        """
            Adds a new fillable to the script

            Args:
                name (str): The name of the fillable
                value (Any): The value to give the fillable

            Returns:
                Any: The value
            """
        self.Fillables[name] = value
        return value

    def add_state(self, name: str, status: str, message: str) -> ScriptState:
        """
            Adds a new state to the script

            Args:
                name (str): The name of the state
                status (str): The status to give the script state
                message (str): The message to give the script state

            Returns:
                State: The new state
            """
        state = ScriptState.State.new(name, status, message)
        self.States[name] = state
        return state

    @staticmethod
    def load(data: dict, name: str)-> "Script":
        """
        Parses the given data into a script

        Args:
            data: The data to parse
            name: The name of the script

        Returns:
            Script: The script created
        """
        # Loads the meta
        if "Meta" in data:
            meta = ScriptMeta.Meta.load(data["Meta"], name)
        else:
            raise ConfigExceptions.ScriptMetaException("No meta given for script", name)
        # Loads the fillable properties set
        fillables = {}
        if "Fillables" in data:
            fillables = data["Fillables"]
        # Loads the script states
        states = {}
        if "States" in data:
            states = data["States"]
            if type(states) != dict:
                raise ConfigExceptions.ScriptException("Invalid state given", name)
            # Loads the state into internal object model
            states = {
                key: ScriptState.State.load(states[key], name) for key in states.keys()
            }
        return Script(
            meta,
            fillables,
            states
        )

    def dump(self) -> dict:
        """
        Dumps the config part into a dictionary
        Returns:
            dict: The data
        """
        data = {
            "Meta": self.Meta.dump()
        }
        if len(self.Fillables) != 0:
            data["Fillables"] = self.Fillables
        if len(self.States) != 0:
            # Turns internal representation of state into dictionary
            data["States"] = {
                key: self.States[key].dump() for key in self.States.keys()
            }
        return data



from unittest import TestCase
import seethrufeeds.core.config_parser as ConfigParser


class TestCoreConfigParser(TestCase):
    def test_create_config(self):
        config = ConfigParser.Config.new("Generic")
        self.assertTrue(config.Header.Scheme_Name == "Generic")

        config.add_script("Foo")
        self.assertTrue(len(config.Scripts) == 1)
        self.assertTrue(config.Scripts["Foo"].Meta.Script_Object_Path == "Scripts.Foo@Foo")
        self.assertTrue(config.Scripts["Foo"].Meta.Script_Output_Path == "Outputs/Foo")

        config.add_api_key("Api_Key", "Access_Token", "Secret")
        self.assertTrue(len(config.Api_Keys) == 1)
        self.assertTrue(config.Api_Keys["Api_Key"].Access_Token == "Access_Token")
        self.assertTrue(config.Api_Keys["Api_Key"].Secret == "Secret")

        config.add_feed("Foo", "Foo", "Foo", "guid")
        self.assertTrue(len(config.Feeds) == 1)
        self.assertTrue(config.Feeds["Foo"].Script == "Foo")
        self.assertTrue(config.Feeds["Foo"].Api_Key == "Foo")
        self.assertTrue(config.Feeds["Foo"].Guid == "guid")

    def test_dump_config(self):
        config = ConfigParser.Config.new("Generic")
        google = config.add_script("Google")
        google.add_fillable("host", "https://google.com")
        config.add_feed("GoogleFeed", "Google", "Default", "abcdefgh-ijkl-mnop-qrst-uvwxyz123456")
        output = config.dump()
        self.assertTrue("Header" in output)

        self.assertTrue("Scripts" in output)
        self.assertTrue("Google" in output["Scripts"])
        self.assertTrue("Meta" in output["Scripts"]["Google"])
        self.assertTrue("Script_Name" in output["Scripts"]["Google"]["Meta"])
        self.assertTrue(output["Scripts"]["Google"]["Meta"]["Script_Name"] == "Google")
        self.assertTrue("Script_Object_Path" in output["Scripts"]["Google"]["Meta"])
        self.assertTrue(output["Scripts"]["Google"]["Meta"]["Script_Object_Path"] == "Scripts.Google@Google")
        self.assertTrue("Script_Output_Path" in output["Scripts"]["Google"]["Meta"])
        self.assertTrue(output["Scripts"]["Google"]["Meta"]["Script_Output_Path"] == "Outputs/Google")

        self.assertTrue("Fillables" in output["Scripts"]["Google"])
        self.assertTrue("host" in output["Scripts"]["Google"]["Fillables"])
        self.assertTrue(output["Scripts"]["Google"]["Fillables"]["host"] == "https://google.com")

        self.assertTrue("Feeds" in output)
        self.assertTrue("GoogleFeed" in output["Feeds"])
        self.assertTrue(output["Feeds"]["GoogleFeed"]['Script'] == "Google")
        self.assertTrue(output["Feeds"]["GoogleFeed"]['Api_Key'] == "Default")
        self.assertTrue(output["Feeds"]["GoogleFeed"]['Guid'] == "abcdefgh-ijkl-mnop-qrst-uvwxyz123456")

    def test_add_scriptstate(self):
        config = ConfigParser.Config.new("Generic")
        google = config.add_script("Google")
        google.add_state("OK", "green", "Test message")

        self.assertTrue(config.Scripts["Google"].States["OK"].Name == "OK")
        self.assertTrue(config.Scripts["Google"].States["OK"].Status == "green")
        self.assertTrue(config.Scripts["Google"].States["OK"].Message == "Test message")

        output = config.dump()
        self.assertTrue(output["Scripts"]["Google"]["States"]["OK"]["Name"] == "OK")
        self.assertTrue(output["Scripts"]["Google"]["States"]["OK"]["Status"] == "green")
        self.assertTrue(output["Scripts"]["Google"]["States"]["OK"]["Message"] == "Test message")



from seethrufeeds.core.exceptions import config as ConfigExceptions

class Feed:
    Script: str
    Api_Key: str
    Guid: str

    def __init__(self, _script, api_key, guid):
        self.Script = _script
        self.Api_Key = api_key
        self.Guid = guid

    @staticmethod
    def new(_script: str, api_key: str, guid: str) -> "Feed":
        """
            Creates a new feed, with given attributes

            Args:
                _script (str): The name of the script that the feed refers to
                api_key (str): The name of the api key to use for the feed
                guid (str): The guid of the feed

            Returns:
                Feed: The new feed
            """
        return Feed(_script, api_key, guid)

    @staticmethod
    def load(data: dict, name: str) -> "Feed":
        """
        Parses the given data into a feed

        Args:
            data: The data to parse
            name: The feed name

        Returns:
            Feed: The new model
        """
        if "Script" not in data:
            raise ConfigExceptions.FeedException("There is no script defined", name)
        if "Api_Key" not in data:
            raise ConfigExceptions.FeedException("There is no api_key defined", name)
        if "Guid" not in data:
            raise ConfigExceptions.FeedException("There is no guid defined", name)
        return Feed(
            data["Script"],
            data["Api_Key"],
            data["Guid"]
        )

    def dump(self) -> dict:
        """
        Dumps the feed into a dictionary

        Returns:
            dict: The dumped data
        """
        return {
            "Script": self.Script,
            "Api_Key": self.Api_Key,
            "Guid": self.Guid
        }



from seethrufeeds.feeds import FeedResult
from seethrufeeds.model.attribution import Attribution

from seethrufeeds.model.scripts.script_state import StatesEnum, State


class ScriptBase(Attribution):
    state: StatesEnum

    def run(self):
        """
        Override this to implement logic for running of the script
        """
        raise NotImplementedError("run method not implemented for script")

    def get_result(self) -> FeedResult:
        """
        Evaluates the script result using the state system, for custom logic, override this

        Returns:
            FeedResult: The result
        """
        if not isinstance(self.state, StatesEnum):
            raise TypeError("state not of type StatesEnum")

        if not isinstance(self.state.value, State):
            raise TypeError("chosen script state not of type State")

        # TODO: Optional override

        # Constructs a new result
        return FeedResult(status=self.state.value.status, message=self.state.value.message)



import unittest

from seethrufeeds.feeds.feed_result import StatusEnum
from seethrufeeds.model.scripts.script_base import ScriptBase
from seethrufeeds.model.scripts.script_state import StatesEnum


class ScriptTestCase(unittest.TestCase):
    def test_script_attribution(self):
        # Creates a test script
        class TestScript(ScriptBase):
            attribute_title = "Test Script"
            attribute_author = "Test Author"

            def run(self): pass

        instance = TestScript()

        self.assertEqual(instance.get_title(), "Test Script")
        self.assertEqual(instance.get_author(), "Test Author")
        self.assertRaises(NotImplementedError, instance.get_description)
        self.assertRaises(NotImplementedError, instance.get_docs_link)
        self.assertRaises(NotImplementedError, instance.get_license_link)
        self.assertRaises(NotImplementedError, instance.get_support_link)

    def test_script_state(self):
        # Creates a test script
        class TestScript(ScriptBase):
            def run(self):
                self.state = StatesEnum.error

        instance = TestScript()
        instance.run()

        result = instance.get_result()
        self.assertEqual(result.status, StatusEnum.red)
        self.assertEqual(result.message, "Error message")

        instance.state = ""
        self.assertRaises(TypeError, instance.get_result)


if __name__ == '__main__':
    unittest.main()



"""
Stores script state
A script can choose to use states rather than an evaluation method,
the script state gets set during the script runtime
Each state has a default status and message, these can be overwritten elsewhere if a custom status or message is needed
"""
from dataclasses import dataclass
from enum import Enum
from seethrufeeds.feeds.feed_result import FeedStatusEnum


@dataclass(frozen=True)
class State:
    status: FeedStatusEnum
    message: str


class StatesEnum(Enum):
    """
    Defines the possible states that a script can have
    """
    ok = State(FeedStatusEnum.green, "")
    warning = State(FeedStatusEnum.amber, "Warning message")
    error = State(FeedStatusEnum.red, "Error message")



from seethrufeeds.model.components.component_base import ComponentBase
from seethrufeeds.model.Properties.properties import FillableProperty, ResultProperty
from pysnmp.hlapi import *


class SNMPWalkToOID(ComponentBase):
    SNMP_HOST = FillableProperty(name="snmp_host", required=True, of_type=str)
    SNMP_PORT = FillableProperty(
        name="snmp_port", default=161, required=True, of_type=int)
    COMMUNITY = FillableProperty(name="community", required=True)

    VALUE = ResultProperty(name="oid_value")

    Component_Title = "SNMP Walk"
    Component_Description = "Connects to an SNMP feed, performs an SNMP Walk, and filters it to get the contents within a specified OID value"
    Component_Author = "SeeThru Networks"
    Component_Owner = "SeeThru Networks"

    def component_execute(self):
        value = ""

        # For every entry in the SNMP table.
        for errorIndication, errorStatus, errorIndex, varBinds in bulkCmd(
                SnmpEngine(),
                CommunityData(self.get_property(self.COMMUNITY)),
                UdpTransportTarget(
                    (self.get_property(self.SNMP_HOST), self.get_property(self.SNMP_PORT))),
                ContextData(),
                0, 50,
                ObjectType(ObjectIdentity(oid)),
                maxCalls=10
        ):
            # Clear the store for the OID key = value pair.
            value = ""

            # Do nothing if there was an SNMP error and return the value ""
            if errorIndication or errorStatus:
                break

            else:
                # Otherwise, take the OID key = value pairs and populate the value store.
                for varBind in varBinds:
                    value = (' = '.join([x.prettyPrint() for x in varBind]))

                # If the entry is the one we're looking for, then remove the "key = "
                # prefix from the value string and return the raw value.
                if search_oid in value:
                    value = value.replace(search_oid + " = ", "")
                    break

            # Then move onto the next OID entry in the SNMP feed.

        # Once we call break, the last value will be outputted
        self.set_property(self.VALUE, value)



import os
import re
from dataclasses import dataclass
from uuid import UUID

from .exceptions import SecretKeyDoesNotExist, InvalidSecretKey, AccessTokenDoesNotExist, InvalidAccessToken


@dataclass(frozen=True)
class ApiKey:
    """
    Stores the api credentials used for the SeeThru api.
    It is recommended to store these in environment variables as they can be easily loaded with ApiKey.from_env
    """
    access_token: UUID
    secret_key: str

    def __post_init__(self):
        """
        Validates the types of the credentials given
        """
        if type(self.access_token) != UUID:
            raise InvalidAccessToken("access_token must be of type UUID")
        if type(self.secret_key) != str:
            raise InvalidSecretKey("secret_key must be of type str")

        # Validates the secret key
        if not re.search("^[0-9a-fA-F]{64}?$", self.secret_key):
            raise InvalidSecretKey("secret_key must be a string of 64 hex characters")

    def get_access_token(self):
        return self.access_token

    def get_secret_key(self):
        return self.secret_key

    @staticmethod
    def from_env(access_token_name: str = "ACCESS_TOKEN", secret_key_name: str = "SECRET_KEY") -> "ApiKey":
        """
        Reads the api key from the environment

        Args:
            access_token_name (str): The name of the access token in the environment
            secret_key_name (str): The name of the secret key in the environment

        Returns:
            ApiKey: A new apikey
        """
        access = os.environ.get(access_token_name, None)
        secret = os.environ.get(secret_key_name, None)
        if access is None:
            raise AccessTokenDoesNotExist()
        if secret is None:
            raise SecretKeyDoesNotExist()

        try:
            access = UUID(access)
        except ValueError:
            raise InvalidAccessToken("access_token must be a valid GUID")

        return ApiKey(access_token=access, secret_key=secret)



from seethrufeeds.model.components.component_base import ComponentBase
from seethrufeeds.model.Properties.properties import FillableProperty, ResultProperty
import requests


class HTTPBase(ComponentBase):
    URL = FillableProperty(name="url", required=True)
    COOKIES = FillableProperty(name="cookies", required=False)
    HEADERS = FillableProperty(name="header", required=False, default=None, of_type=dict)
    RESPONSE = ResultProperty(name="response")
    STATUS_CODE = ResultProperty(name="status_code")
    RESPONSE_CONTENT = ResultProperty(name="response_content")
    RESPONSE_URL = ResultProperty(name="response_url")

    Component_Title = "HTTP Component"
    Component_Description = "This component provides a wrapper over the requests http methods to make them follow the component design rules"
    Component_Author = "SeeThru Networks"
    Component_Owner = "SeeThru Networks"


class HTTPGet(HTTPBase):
    def component_execute(self):
        response = requests.get(self.get_property(
            HTTPBase.URL), cookies=self.get_property(HTTPBase.COOKIES), headers=self.get_property(HTTPBase.HEADERS))

        self.set_property(HTTPGet.RESPONSE, response)
        self.set_property(HTTPGet.STATUS_CODE, response.status_code)
        self.set_property(HTTPGet.COOKIES, response.cookies)
        self.set_property(HTTPGet.RESPONSE_CONTENT, response.text)
        self.set_property(HTTPGet.RESPONSE_URL, response.url)


class HTTPPost(HTTPBase):
    DATA = FillableProperty(name="data")
    JSON = FillableProperty(name="json", of_type=dict, required=False)
    CONTENT_TYPE = FillableProperty(
        name="content_type", default="application/x-www-form-urlencoded")

    def component_execute(self):
        if self.get_property(HTTPBase.HEADERS) is None:
            self.set_property(HTTPBase.HEADERS, {'Content-Type': self.get_property(self.CONTENT_TYPE)})

        response = requests.post(
            self.get_property(HTTPPost.URL),
            cookies=self.get_property(HTTPPost.COOKIES),
            data=self.get_property(HTTPPost.DATA),
            json=self.get_property(HTTPPost.JSON),
            headers=self.get_property(HTTPBase.HEADERS)
        )
        self.set_property(HTTPPost.RESPONSE, response)
        self.set_property(HTTPPost.STATUS_CODE, response.status_code)
        self.set_property(HTTPPost.RESPONSE_CONTENT, response.text)
        self.set_property(HTTPPost.RESPONSE_URL, response.url)



import json
from dataclasses import dataclass
from json import JSONDecodeError
from uuid import UUID

import requests
from requests import RequestException

from seethrufeeds.feeds.api_key import ApiKey
from seethrufeeds.feeds.exceptions import InvalidFeedGuid, InvalidApiKey, ApiError, MalformedApiResponse, \
    ApiUnexpectedError, ApiInvalidGuid, ApiKeyInvalidPermissions, ApiKeyTimeout, ApiInvalidFeedResult, \
    ApiKeyUnauthorized
from seethrufeeds.feeds.feed_result import FeedResult


def match_error_api_message_to_exception(message: str):
    """
    Matches the message given by the api to a specific exception
    Args:
        message (str): The message to match
    """
    if message == "An unexpected error occurred.":
        raise ApiUnexpectedError()
    elif message == "Please specify a valid Feed GUID.":
        raise ApiInvalidGuid()
    elif message == "Please use an authorised API key.":
        raise ApiKeyUnauthorized()
    elif message == "You do not have permission to update the selected feed.":
        raise ApiKeyInvalidPermissions()
    elif message == "This feed has already been updated within the last 30 seconds.":
        raise ApiKeyTimeout()
    elif message == "Invalid result color specified.":
        raise ApiInvalidFeedResult("Invalid feed result status")
    elif message == "Invalid timestamp specified.":
        raise ApiInvalidFeedResult("Invalid feed result timestamp")
    elif message == "Invalid version number specified.":
        raise ApiInvalidFeedResult("Invalid feed version")


@dataclass(frozen=True)
class Feed:
    """
    Represents a feed on SeeThru, every feed requires a feed guid (provided by SeeThru), an api key and a version,
    the version provided will be the version that feed results are pushed with
    """
    feed_guid: UUID
    api_key: ApiKey
    version: str = "1"

    def __post_init__(self):
        """
        Validates the types of the feed fields
        """
        if type(self.feed_guid) != UUID:
            raise InvalidFeedGuid("feed_guid must be of type UUID")
        if type(self.api_key) != ApiKey:
            raise InvalidApiKey("api_key must be of type ApiKey")
        if type(self.version) != str:
            raise TypeError("version must be of type str")

    def get_feed_guid(self):
        return self.feed_guid

    def get_api_key(self):
        return self.api_key

    def push(self, result: FeedResult):
        """
        Pushes the result to SeeThru under the current feed, the version of the feed is pushed with the result

        Args:
            result (FeedResult): The result to push to SeeThru
        """
        headers = {
            "X-Access-Token": str(self.api_key.access_token),
            "X-secret": self.api_key.secret_key
        }
        data = {
            "guid": str(self.feed_guid),
            "color": result.get_status_value(),
            "message": result.message,
            "timestamp": result.get_timestamp_str(),
            "version": self.version
        }

        try:
            response = requests.post(
                f"https://api.seethrunetworks.com/feed/{str(self.feed_guid)}/update",
                data=data,
                headers=headers)
        except RequestException as e:
            raise e

        try:
            response_json = json.loads(response.content)
            if "success" not in response_json:
                raise MalformedApiResponse()
            # Only matches the error for an unsuccessful post
            if not response_json["success"]:
                if "message" not in response_json:
                    raise MalformedApiResponse()

                try:
                    match_error_api_message_to_exception(response_json["message"])
                except ApiError as e:
                    raise e

        except JSONDecodeError:
            raise MalformedApiResponse()



from setuptools import setup, find_packages

# setup()

setup(
    name="seethrufeeds",
    version="0.1.4",
    author="SeeThru Networks",
    author_email="aidan@seethrunetworks.com",
    description="The SeeThruNetworks feed framework",
    license="MIT",
    url="https://github.com/SeeThru-Networks/Python-Feeds",
    project_urls={
        "Website": "https://www.seethrunetworks.com/"
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only"
    ],
    packages=find_packages(),
    install_requires=['requests', "python-dotenv", "toml", "mysql-connector-python", "py-zabbix", "pydantic"],
    python_requires=">=3.9",
    keywords=["seethru", "seethrunetworks", "feed"],
    entry_points={
        "console_scripts": [
                "SeeThru_Feeds=SeeThru_Feeds.Core.SeeThruFeed:exec",
                "seethrufeeds=SeeThru_Feeds.Core.SeeThruFeed:exec"
            ]
    }


)



from datetime import datetime, timezone
from enum import Enum


class FeedStatusEnum(Enum):
    green = "green"
    amber = "amber"
    red = "red"
    default = "default"

    # Links to colours
    ok = green
    warning = amber
    error = red


class FeedResult:
    """
    Stores a feed result that can be transferred to SeeThru via the api,
    the timestamp provided can be any timezone but will be converted to utc upon pushing
    """
    # Containers of the true values
    _status: FeedStatusEnum
    _message: str
    _timestamp: datetime

    def __init__(self, status: FeedStatusEnum, message: str, timestamp: datetime = None):
        """
        Args:
            status (FeedStatusEnum): The status of the result
            message (str): The message of the result
            timestamp (datetime): The timestamp of the result, defaults to utc now
        """
        self.status = status
        self.message = message
        if timestamp is None:
            self.timestamp = datetime.now()
        else:
            self.timestamp = timestamp

    @property
    def status(self) -> FeedStatusEnum:
        return self._status

    @status.setter
    def status(self, status: FeedStatusEnum):
        if status not in FeedStatusEnum:
            raise TypeError("status must be of type FeedStatusEnum")
        self._status = status

    def get_status(self) -> FeedStatusEnum:
        return self.status

    def get_status_value(self) -> str:
        """
        Gets the value attached to the status enum

        Returns:
            The colour value
        """
        return self.status.value

    @property
    def message(self) -> str:
        return self._message

    @message.setter
    def message(self, message: str):
        if type(message) != str:
            raise TypeError("message must be of type string")
        self._message = message

    def get_message(self) -> str:
        return self.message

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp: datetime):
        """
        Converts the given timezone to utc and sets it

        Args:
            timestamp (datetime): The timezone to be converted to utc
        """
        if type(timestamp) != datetime:
            raise TypeError("timestamp must be of type datetime")
        self._timestamp = timestamp.astimezone(tz=timezone.utc)

    def get_timestamp(self) -> datetime:
        return self.timestamp

    def get_timestamp_str(self) -> str:
        """
        Returns a %Y-%m-%d %H:%M:%S formatted timestamp

        Returns:
            str: The formatted timestamp
        """
        return self._timestamp.astimezone(tz=timezone.utc).strftime("%Y-%m-%d %H:%M:%S")




from seethrufeeds.core.exceptions import config as ConfigExceptions

class ApiKey:
    Access_Token: str
    Secret: str

    def __init__(self, access_token, secret):
        self.Access_Token = access_token
        self.Secret = secret

    @staticmethod
    def new(access_token: str, secret: str) -> "ApiKey":
        """
            Creates a new api key, with given attributes

            Args:
                access_token (str): The access token
                secret (str): The secret key

            Returns:
                ApiKey: The new api key
            """
        return ApiKey(access_token, secret)

    @staticmethod
    def load(data: dict, name: str) -> "ApiKey":
        """
        Parses the given data into an api key

        Args:
            data: The data to parse
            name: The name of the api key

        Returns:
            ApiKey: The new api key
        """
        if "Access_Token" not in data:
            raise ConfigExceptions.ApiKeyException(f"There is no access_token", name)
        if "Secret" not in data:
            raise ConfigExceptions.ApiKeyException(f"There is no secret", name)
        return ApiKey(
            data["Access_Token"],
            data["Secret"]
        )

    def dump(self) -> dict:
        """
        Dumps the api key into a dictionary

        Returns:
            dict: The dumped data
        """
        return {
            "Access_Token": self.Access_Token,
            "Secret": self.Secret
        }



class InvalidApiKey(TypeError):
    pass


class SecretKeyDoesNotExist(Exception):
    pass


class InvalidSecretKey(TypeError):
    pass


class AccessTokenDoesNotExist(Exception):
    pass


class InvalidAccessToken(TypeError):
    pass


class InvalidFeedGuid(TypeError):
    pass


class ApiError(Exception):
    pass


class MalformedApiResponse(ApiError):
    pass


class ApiUnexpectedError(ApiError):
    pass


class ApiInvalidGuid(ApiError):
    pass


class ApiKeyUnauthorized(ApiError):
    pass


class ApiKeyInvalidPermissions(ApiError):
    pass


class ApiKeyTimeout(ApiError):
    pass


class ApiInvalidFeedResult(ApiError):
    pass
